@@ -1,10 +1,8 @@
-from .dataset import Dataset
 from .data import (
     ExperimentReferenceIndex,
     ReferenceIndex,
     serialize_for_hashing,
-    compute_md5_hash,
-    compute_experiment_reference_index,
+    compute_sha256_hash,
 )
 
 data = [
@@ -12,9 +10,7 @@ data = [
     "ReferenceIndex",
     "serialize_for_hashing",
     "compute_md5_hash",
-    "compute_experiment_reference_index",
 ]
 
-dataset = ["Dataset"]
 
-__all__ = dataset + data
+__all__ = data
