@@ -3,12 +3,8 @@
 # https://github.com/Mjvolk3/torchcell/tree/main/torchcell/datamodels/pydantic.py
 # Test file: torchcell/datamodels/test_pydantic.py
 
-import json
-from typing import Any, List, Optional
 
-import matplotlib.pyplot as plt
-import networkx as nx
-from pydantic import BaseModel, ConfigDict
+from pydantic import BaseModel
 
 
 class ModelStrict(BaseModel):
