@@ -10,31 +10,26 @@ import os.path as osp
 import pickle
 import zipfile
 from collections.abc import Callable
-import torch
+
+# import torch
 import lmdb
 import numpy as np
 import pandas as pd
 from torch_geometric.data import download_url
 from tqdm import tqdm
-
-from torchcell.data import (
-    Dataset,
-    ExperimentReferenceIndex,
-    compute_experiment_reference_index,
-)
-from torchcell.datamodels import (
+from torchcell.dataset import Dataset, compute_experiment_reference_index
+from torchcell.data import ExperimentReferenceIndex
+from torchcell.datamodels.schema import (
     BaseEnvironment,
-    BaseGenotype,
-    DeletionGenotype,
+    Genotype,
     FitnessExperiment,
     FitnessExperimentReference,
     FitnessPhenotype,
-    InterferenceGenotype,
     Media,
     ReferenceGenome,
     SgaKanMxDeletionPerturbation,
-    SgdAllelePerturbation,
-    SgdTsAllelePerturbation,
+    SgaAllelePerturbation,
+    SgaTsAllelePerturbation,
     Temperature,
 )
 from torchcell.sequence import GeneSet
@@ -230,7 +225,7 @@ class SmfKuzmin2018Dataset(Dataset):
         if row["smf_type"] == "query_smf":
             # Query
             if "KanMX_deletion" in row["query_perturbation_type"]:
-                genotype = DeletionGenotype(
+                genotype = Genotype(
                     perturbation=SgaKanMxDeletionPerturbation(
                         systematic_gene_name=row["Query systematic name no ho"],
                         perturbed_gene_name=row["Query allele name no ho"],
@@ -239,8 +234,8 @@ class SmfKuzmin2018Dataset(Dataset):
                 )
 
             elif "allele" in row["query_perturbation_type"]:
-                genotype = BaseGenotype(
-                    perturbation=SgdAllelePerturbation(
+                genotype = Genotype(
+                    perturbation=SgaAllelePerturbation(
                         systematic_gene_name=row["Query systematic name no ho"],
                         perturbed_gene_name=row["Query allele name no ho"],
                         strain_id=row["Query strain ID"],
@@ -250,7 +245,7 @@ class SmfKuzmin2018Dataset(Dataset):
         elif row["smf_type"] == "array_smf":
             # Array
             if "KanMX_deletion" in row["array_perturbation_type"]:
-                genotype = DeletionGenotype(
+                genotype = Genotype(
                     perturbation=SgaKanMxDeletionPerturbation(
                         systematic_gene_name=row["Array systematic name"],
                         perturbed_gene_name=row["Array allele name"],
@@ -259,8 +254,8 @@ class SmfKuzmin2018Dataset(Dataset):
                 )
 
             elif "allele" in row["array_perturbation_type"]:
-                genotype = BaseGenotype(
-                    perturbation=SgdAllelePerturbation(
+                genotype = Genotype(
+                    perturbation=SgaAllelePerturbation(
                         systematic_gene_name=row["Array systematic name"],
                         perturbed_gene_name=row["Array allele name"],
                         strain_id=row["Array strain ID"],
@@ -269,8 +264,8 @@ class SmfKuzmin2018Dataset(Dataset):
 
             # Only array has ts
             elif "temperature_sensitive" in row["array_perturbation_type"]:
-                genotype = InterferenceGenotype(
-                    perturbation=SgdTsAllelePerturbation(
+                genotype = Genotype(
+                    perturbation=SgaTsAllelePerturbation(
                         systematic_gene_name=row["Array systematic name"],
                         perturbed_gene_name=row["Array allele name"],
                         strain_id=row["Array strain ID"],
@@ -351,18 +346,15 @@ class SmfKuzmin2018Dataset(Dataset):
         if self.env is None:
             self._init_db()
 
-        # Handling boolean index tensors or numpy arrays
-        if isinstance(idx, (list, np.ndarray, torch.Tensor)):
+        # Handling boolean index arrays or numpy arrays
+        if isinstance(idx, (list, np.ndarray)):
             if isinstance(idx, list):
                 idx = np.array(idx)
-            if isinstance(idx, np.ndarray) and idx.dtype == np.bool:
+            if idx.dtype == np.bool_:
                 idx = np.where(idx)[0]
-            elif isinstance(idx, torch.Tensor) and idx.dtype == torch.bool:
-                idx = idx.nonzero(as_tuple=False).squeeze(1)
 
-        if isinstance(idx, (np.ndarray, list, torch.Tensor)):
-            # If idx is a list/array/tensor of indices, return a list of data objects
-            return [self.get_single_item(i.item()) for i in idx]
+            # If idx is a list/array of indices, return a list of data objects
+            return [self.get_single_item(i) for i in idx]
         else:
             # Single item retrieval
             return self.get_single_item(idx)
@@ -608,6 +600,9 @@ class DmfKuzmin2018Dataset(Dataset):
         self.reference_phenotype_std = df[
             "Combined mutant fitness standard deviation"
         ].mean()
+        # replace delta symbol for neo4j import
+        df = df.replace("'", "_prime", regex=True)
+        df = df.replace("Δ", "_delta", regex=True)
         df = df.reset_index(drop=True)
         return df
 
@@ -622,7 +617,7 @@ class DmfKuzmin2018Dataset(Dataset):
         # Query...
         if "KanMX_deletion" in row["query_perturbation_type_no_ho"]:
             genotype.append(
-                DeletionGenotype(
+                Genotype(
                     perturbation=SgaKanMxDeletionPerturbation(
                         systematic_gene_name=row["Query systematic name no ho"],
                         perturbed_gene_name=row["Query allele name no ho"],
@@ -632,8 +627,8 @@ class DmfKuzmin2018Dataset(Dataset):
             )
         elif "allele" in row["query_perturbation_type_no_ho"]:
             genotype.append(
-                BaseGenotype(
-                    perturbation=SgdAllelePerturbation(
+                Genotype(
+                    perturbation=SgaAllelePerturbation(
                         systematic_gene_name=row["Query systematic name no ho"],
                         perturbed_gene_name=row["Query allele name no ho"],
                         strain_id=row["Query strain ID"],
@@ -644,8 +639,8 @@ class DmfKuzmin2018Dataset(Dataset):
         # Array - only array has ts
         if "temperature_sensitive" in row["array_perturbation_type"]:
             genotype.append(
-                InterferenceGenotype(
-                    perturbation=SgdTsAllelePerturbation(
+                Genotype(
+                    perturbation=SgaTsAllelePerturbation(
                         systematic_gene_name=row["Array systematic name"],
                         perturbed_gene_name=row["Array allele name"],
                         strain_id=row["Array strain ID"],
@@ -654,7 +649,7 @@ class DmfKuzmin2018Dataset(Dataset):
             )
         elif "KanMX_deletion" in row["array_perturbation_type"]:
             genotype.append(
-                DeletionGenotype(
+                Genotype(
                     perturbation=SgaKanMxDeletionPerturbation(
                         systematic_gene_name=row["Array systematic name"],
                         perturbed_gene_name=row["Array allele name"],
@@ -1000,6 +995,9 @@ class TmfKuzmin2018Dataset(Dataset):
         self.reference_phenotype_std = df[
             "Combined mutant fitness standard deviation"
         ].mean()
+        # replace delta symbol for neo4j import
+        df = df.replace("'", "_prime", regex=True)
+        df = df.replace("Δ", "_delta", regex=True)
         df = df.reset_index(drop=True)
         return df
 
@@ -1015,7 +1013,7 @@ class TmfKuzmin2018Dataset(Dataset):
         # Query 1
         if "KanMX_deletion" in row["query_perturbation_type_1"]:
             genotype.append(
-                DeletionGenotype(
+                Genotype(
                     perturbation=SgaKanMxDeletionPerturbation(
                         systematic_gene_name=row["Query systematic name_1"],
                         perturbed_gene_name=row["Query allele name_1"],
@@ -1025,8 +1023,8 @@ class TmfKuzmin2018Dataset(Dataset):
             )
         elif "allele" in row["query_perturbation_type_1"]:
             genotype.append(
-                BaseGenotype(
-                    perturbation=SgdAllelePerturbation(
+                Genotype(
+                    perturbation=SgaAllelePerturbation(
                         systematic_gene_name=row["Query systematic name_1"],
                         perturbed_gene_name=row["Query allele name_1"],
                         strain_id=row["Query strain ID"],
@@ -1036,7 +1034,7 @@ class TmfKuzmin2018Dataset(Dataset):
         # Query 2
         if "KanMX_deletion" in row["query_perturbation_type_2"]:
             genotype.append(
-                DeletionGenotype(
+                Genotype(
                     perturbation=SgaKanMxDeletionPerturbation(
                         systematic_gene_name=row["Query systematic name_2"],
                         perturbed_gene_name=row["Query allele name_2"],
@@ -1046,8 +1044,8 @@ class TmfKuzmin2018Dataset(Dataset):
             )
         elif "allele" in row["query_perturbation_type_2"]:
             genotype.append(
-                BaseGenotype(
-                    perturbation=SgdAllelePerturbation(
+                Genotype(
+                    perturbation=SgaAllelePerturbation(
                         systematic_gene_name=row["Query systematic name_2"],
                         perturbed_gene_name=row["Query allele name_2"],
                         strain_id=row["Query strain ID"],
@@ -1057,8 +1055,8 @@ class TmfKuzmin2018Dataset(Dataset):
         # Array - only array has ts
         if "temperature_sensitive" in row["array_perturbation_type"]:
             genotype.append(
-                InterferenceGenotype(
-                    perturbation=SgdTsAllelePerturbation(
+                Genotype(
+                    perturbation=SgaTsAllelePerturbation(
                         systematic_gene_name=row["Array systematic name"],
                         perturbed_gene_name=row["Array allele name"],
                         strain_id=row["Array strain ID"],
@@ -1067,7 +1065,7 @@ class TmfKuzmin2018Dataset(Dataset):
             )
         elif "KanMX_deletion" in row["array_perturbation_type"]:
             genotype.append(
-                DeletionGenotype(
+                Genotype(
                     perturbation=SgaKanMxDeletionPerturbation(
                         systematic_gene_name=row["Array systematic name"],
                         perturbed_gene_name=row["Array allele name"],
